import os
import sys
from typing import Optional

import requests

PUSHCUTS_URL = os.environ.get("PUSHCUTS_URL")


class PushcutsError(Exception):
    pass


def pushcut(title: str, text: str, pushcuts_url: Optional[str] = None):
    """
    Setting pushcuts_url overrides the value from environment variable
    """
    if pushcuts_url is None:
        if PUSHCUTS_URL is None:
            raise PushcutsError("Setting a pushcuts URL is required for notification")
        else:
            url = PUSHCUTS_URL
    else:
        url = pushcuts_url
    requests.get(url, {"title": title, "text": text})


def stdin_input():
    for line in sys.stdin:
        return line.strip()


def pushcuts_main(title: Optional[str] = None, text: Optional[str] = None):
    if title is not None and text is not None:
        pushcut(title, text)
    elif title is None and text is not None:
        pushcut(stdin_input(), text)
    elif title is not None and text is None:
        pushcut(title, stdin_input())
    else:
        pushcut("Terminal Notification", stdin_input())



import logging

from rich.logging import RichHandler

logging.basicConfig(
    format="%(message)s",
    level=logging.INFO,
    handlers=[RichHandler(rich_tracebacks=True)],
)


def get_logger(name: str):
    return logging.getLogger(name)



from typing import Callable, Dict, Iterable, List, TypeVar

T = TypeVar("T")
K = TypeVar("K")


def group_by(func: Callable[[T], K], iterable: Iterable[T]) -> Dict[K, List[T]]:
    entries: Dict[K, List[T]] = {}
    for e in iterable:
        key = func(e)
        if key in entries:
            entries[key].append(e)
        else:
            entries[key] = [e]

    return entries



from pathlib import Path
from typing import List, Union

import altair as alt
import altair_saver

from pedroai.io import safe_file


def save_chart(
    chart: alt.Chart, base_path: Union[Path, str], filetypes: List[str], method=None
):
    base_path = str(base_path)
    for t in filetypes:
        path = base_path + "." + t
        if method == "node" and t in ("svg", "pdf"):
            method = "node"
        else:
            method = None
        altair_saver.save(chart, safe_file(path), method=method)



import os
import subprocess
from typing import Dict, List, Union

import toml
from pydantic import BaseModel

from pedroai.log import get_logger

log = get_logger(__name__)

VERSION = "0.1.0"


class Options(BaseModel):
    create_dirs: bool
    directories: List[str]


class DownloadSpec(BaseModel):
    url: str
    checksum: str


class Job(BaseModel):
    name: str
    command: str
    check: str


class Config(BaseModel):
    version: str
    options: Options
    downloads: Dict[str, Union[str, DownloadSpec]]
    jobs: Dict[str, Job]


def _download_file(remote_file: str, local_file: str, dry_run: bool = False):
    if not dry_run:
        subprocess.run(
            f"curl -f --create-dirs -o {local_file} {remote_file}",
            shell=True,
            check=True,
        )


def download_file(
    remote_file: str, local_file: str, overwrite=False, dry_run: bool = False
):
    if os.path.exists(local_file):
        if overwrite:
            log.info("Overwriting, Downloading %s to %s", remote_file, local_file)
            _download_file(remote_file, local_file, dry_run=dry_run)
        else:
            log.info("File exists, skipping download of: %s", local_file)
    else:
        log.info("Downloading %s to %s", remote_file, local_file)
        _download_file(remote_file, local_file, dry_run=dry_run)


def main(
    config_file: str = "files.toml",
    overwrite: bool = False,
    download: bool = True,
    dry_run: bool = False,
    enforce_version: bool = True,
):
    log.info("Configuration")
    log.info("config_file: %s", config_file)
    log.info("overwrite: %s", overwrite)
    log.info("download: %s", download)
    log.info("dry_run: %s", dry_run)
    log.info("enforce_version: %s", enforce_version)
    with open(config_file) as f:
        log.info("Validating: %s", config_file)
        config = Config(**toml.load(f))
        if config.version != VERSION:
            log.warning(
                "Mismatched configuration versions: %s (script) vs %s (config)",
                VERSION,
                config.version,
            )
            if enforce_version:
                raise ValueError("Configuration version mismatch not allowed")

    if config.options.create_dirs:
        log.info("Creating Directories")
        for directory in config.options.directories:
            log.info("Creating: %s", directory)
            if not dry_run:
                os.makedirs(directory, exist_ok=True)

    if download:
        for local_file, remote_file in config.downloads.items():
            if isinstance(remote_file, str):
                download_file(
                    remote_file, local_file, overwrite=overwrite, dry_run=dry_run
                )
            elif isinstance(remote_file, DownloadSpec):
                download_file(
                    remote_file.url, local_file, overwrite=overwrite, dry_run=dry_run
                )
            else:
                raise ValueError(f"Invalid type for remote_file: {type(remote_file)}")

    for job in config.jobs.values():
        log.info("Running: %s", job.command)
        if not dry_run:
            if os.path.exists(job.check):
                log.info("Check path exists, skipping: %s", job.check)
            else:
                subprocess.run(job.command, shell=True, check=True)



import json
import os
import subprocess
import sys
from pathlib import Path
from typing import Any, List, Union

import requests
import simdjson
from pydantic import BaseModel
from rich.console import Console

console = Console()


def shell(command: str):
    subprocess.run(command, shell=True, check=True)


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def read_json(path: Union[str, Path]):
    """
    Read a json file from a string path
    """
    with open(path) as f:
        return simdjson.load(f)  # type: ignore


def write_json(path: Union[str, Path], obj: Any):
    """
    Write an object to a string path as json.
    If the object is a pydantic model, export it to json
    """
    if isinstance(obj, BaseModel):
        with open(path, "w") as f:
            f.write(obj.json())
    else:
        with open(path, "w") as f:
            json.dump(obj, f)


def _read_jsonlines_list(path: Union[str, Path]):
    """
    Read a jsonlines file into memory all at once
    """
    parser = simdjson.Parser()
    out = []
    with open(path) as f:
        for line in f:
            out.append(parser.parse(line, recursive=True))
    return out


def _read_jsonlines_lazy(path: Union[str, Path]):
    """
    Lazily return the contents of a jsonlines file
    """
    parser = simdjson.Parser()
    with open(path) as f:
        for line in f:
            yield parser.parse(line, recursive=True)


def read_jsonlines(path: Union[str, Path], lazy: bool = False):
    """
    Read a jsonlines file as a list/iterator of json objects
    """
    if lazy:
        return _read_jsonlines_lazy(path)
    else:
        return _read_jsonlines_list(path)


def write_jsonlines(path: Union[str, Path], elements: List[Any]):
    """
    Write a list of json serialiazable objects to the path given
    """
    with open(path, "w") as f:
        for e in elements:
            f.write(json.dumps(e))
            f.write("\n")


def download(remote_path, local_path):
    console.log(f"Downloading {remote_path} to {local_path}")
    response = requests.get(remote_path, stream=True)
    with open(local_path, "w") as f:
        for data in response.iter_content():
            f.write(data)


class requires_file:
    def __init__(self, path: Union[str, Path]):
        self._path = path

    def __call__(self, f):
        if os.path.exists(self._path):
            return f
        else:

            def nop(*args, **kwargs):  # pylint: disable=unused-argument
                console.log("File missing, skipping function: path=", self._path)

            return nop


class requires_files:
    def __init__(self, paths: List[Union[str, Path]]):
        self._paths = paths

    def __call__(self, f):
        missing_files = []
        for path in self._paths:
            if not os.path.exists(path):
                missing_files.append(path)

        if len(missing_files) > 0:

            def nop(*args, **kwargs):  # pylint: disable=unused-argument
                console.log("Files missing, skipping function: paths=", missing_files)

            return nop
        else:
            return f


def safe_file(path: Union[str, Path]) -> Union[str, Path]:
    """
    Ensure that the path to the file exists, then return the path.

    For example, if the path passed in is /home/entilzha/stuff/stuff/test.txt,
    this function will run the equivalent of mkdir -p /home/entilzha/stuff/stuff/
    """
    os.makedirs(os.path.dirname(path), exist_ok=True)
    return path



from plotnine import (
    element_blank,
    element_line,
    element_rect,
    element_text,
    theme,
    theme_light,
)

DISCRETE_COLORS = ["#1ECBE0", "#E0331E", "#6AE01E", "#941EE0"]


class theme_pedroai(theme_light):
    """
    A theme similar to :class:`theme_linedraw` but with light grey
    lines and axes to direct more attention towards the data.
    Parameters
    ----------
    base_size : int, optional
        Base font size. All text sizes are a scaled versions of
        the base font size. Default is 11.
    base_family : str, optional
        Base font family.
    """

    def __init__(self, base_size=11, base_family="DejaVu Sans"):
        theme_light.__init__(self, base_size, base_family)
        self.add_theme(
            theme(
                text=element_text(size=14),
                axis_ticks=element_line(color="#DDDDDD", size=0.5),
                panel_border=element_rect(fill="None", color="#838383", size=1),
                panel_spacing=0.40,
                strip_background=element_rect(fill="#DDDDDD", color="#838383", size=1),
                strip_text_x=element_text(color="black"),
                strip_text_y=element_text(color="black", angle=-90),
                legend_key=element_blank(),
                legend_position="top",
                plot_margin=0,
            ),
            inplace=True,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pedroai']

package_data = \
{'': ['*']}

install_requires = \
['altair-saver>=0.5.0,<0.6.0',
 'altair>=4.2.0,<5.0.0',
 'plotnine>=0.9,<0.10',
 'pydantic>=1.9.0,<2.0.0',
 'pysimdjson[dev]>=4.0.3,<5.0.0',
 'requests>=2.27.1,<3.0.0',
 'rich>=12.4.1,<13.0.0',
 'scipy>=1.7.3,<2.0.0',
 'toml>=0.10.2,<0.11.0',
 'typer>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['pedroai = pedroai.main:cli']}

setup_kwargs = {
    'name': 'pedroai',
    'version': '0.4.0',
    'description': '',
    'long_description': None,
    'author': 'Pedro Rodriguez',
    'author_email': 'me@pedro.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.12',
}


setup(**setup_kwargs)



from abc import ABC, abstractmethod
from typing import Callable, Generic, Optional, TypeVar, Union

T = TypeVar("T")
E = TypeVar("E")
U = TypeVar("U")
F = TypeVar("F")


class ResultException(Exception):
    pass


class AbstractResult(ABC, Generic[T, E]):
    @abstractmethod
    def is_ok(self) -> bool:
        pass

    @abstractmethod
    def is_err(self) -> bool:
        pass

    @abstractmethod
    def ok(self) -> Optional[T]:
        pass

    @abstractmethod
    def err(self) -> Optional[E]:
        pass

    @abstractmethod
    def map(self, f: Callable[[T], U]) -> "Result[U, E]":
        pass

    @abstractmethod
    def map_err(self, f: Callable[[E], F]) -> "Result[T, F]":
        pass

    @abstractmethod
    def unwrap_or(self, val: T) -> T:
        pass

    @abstractmethod
    def unwrap(self) -> T:
        pass

    @abstractmethod
    def expect(self, msg: str) -> T:
        pass

    @abstractmethod
    def unwrap_err(self) -> E:
        pass

    @abstractmethod
    def expect_err(self, msg: str) -> E:
        pass


class Ok(AbstractResult, Generic[T]):
    def __init__(self, value: T):
        self._value = value

    def __repr__(self):
        return f"Ok({self._value})"

    def is_ok(self) -> bool:
        return True

    def is_err(self) -> bool:
        return False

    def ok(self) -> Optional[T]:
        return self._value

    def err(self) -> Optional[E]:
        return None

    def map(self, f: Callable[[T], U]) -> "Result[U, E]":
        return Ok(f(self._value))

    def map_err(self, f: Callable[[E], F]) -> "Result[T, F]":
        return self

    def unwrap_or(self, val: T) -> T:
        return self._value

    def unwrap(self) -> T:
        return self._value

    def unwrap_err(self) -> E:
        raise ResultException(f"{self._value}")

    def expect(self, msg: str) -> T:
        return self._value

    def expect_err(self, msg: str) -> E:
        raise ResultException(f"{msg}: {self._value}")


class Err(AbstractResult, Generic[E]):
    def __init__(self, err_value: E):
        self._err_value = err_value

    def __repr__(self):
        return f"Err({self._err_value})"

    def is_ok(self) -> bool:
        return False

    def is_err(self) -> bool:
        return True

    def ok(self) -> Optional[T]:
        return None

    def err(self) -> Optional[E]:
        return self._err_value

    def map(self, f: Callable[[T], U]) -> "Result[U, E]":
        return self

    def map_err(self, f: Callable[[E], F]) -> "Result[T, F]":
        return Err(f(self._err_value))

    def unwrap_or(self, val: T) -> T:
        return val

    def unwrap(self) -> T:
        raise ResultException(f"{self._err_value}")

    def unwrap_err(self) -> E:
        return self._err_value

    def expect(self, msg: str) -> T:
        raise ResultException(f"{msg}: {self._err_value}")

    def expect_err(self, msg: str) -> E:
        return self._err_value


Result = Union[Ok[T], Err[E]]  # pylint: disable=unsubscriptable-object



import typer

from pedroai import download, notifications

cli = typer.Typer()
cli.command(name="download")(download.main)
cli.command(name="pushcuts")(notifications.pushcuts_main)


@cli.command()
def nop():
    pass


if __name__ == "__main__":
    cli()
